#define ColorStone    &hff7700
#define ColorCrystal  &h0000ff
#define ColorCreature &h00ff00
#define MAXEFFECTS    4

enum TEffectType
  none = 0
  changeOwnStone, changeOwnCrystal, changeOwnCreature, changeOwnStoneProd, changeOwnCrystalProd, changeOwnCreatureProd
  changeOppStone, changeOppCrystal, changeOppCreature, changeOppStoneProd, changeOppCrystalProd, changeOppCreatureProd
  changeBothStone, changeBothCrystal, changeBothCreature, changeBothStoneProd, changeBothCrystalProd, changeBothCreatureProd
  stealStone, stealCrystal, stealCreature, stealStoneProd, stealCrystalProd, stealCreatureProd
  changeOwnWall, changeOppWall, changeBothWall, changeOwnTower, changeOppTower, changeBothTower
  ownDamage, oppDamage, bothDamage
  damageOwnWall, damageOppWall, damageBothWall, damageOwnTower, damageOppTower, damageBothTower
end enum
enum TCond
  none = 0
  ownStone,  ownCrystal,  ownCreature,  ownStoneProd,  ownCrystalProd,  ownCreatureProd,  ownWall,  ownTower
  oppStone,  oppCrystal,  oppCreature,  oppStoneProd,  oppCrystalProd,  oppCreatureProd,  oppWall,  oppTower
  diffStone, diffCrystal, diffCreature, diffStoneProd, diffCrystalProd, diffCreatureProd, diffWall, diffTower
  diffOwnBuilding, diffOppBuilding
end enum
enum TComp
  none = 0
  equal, greater, lower
end enum
namespace Special
  dim as integer nothing = 0, again = 1, stone = 2, crystal = 4, creature = 6
end namespace

type TGameSettings
  as integer waitBeforeDisplay, waitAfterDisplay
  as integer startPStone, startPCrystal, startPCreature
  as integer startStone, startCrystal, startCreature
  as integer maxStone, maxCrystal, maxCreature
  as integer startTower, startWall, maxTower
  as string*256 fillTower, fillWall
end type
dim shared as TGameSettings setting

type TPlayer
  as integer pStone, pCrystal, pCreature                 ' production
  as integer stone, crystal, creature                    ' resources
  as integer wall, tower, card(1 to 6)
  declare constructor
  declare sub production
end type
constructor TPlayer
  this.pStone    = setting.startPStone
  this.pCrystal  = setting.startPCrystal
  this.pCreature = setting.startPCreature
  this.stone     = setting.startStone
  this.crystal   = setting.startCrystal
  this.creature  = setting.startCreature
  this.tower     = setting.startTower
  this.wall      = setting.startWall
end constructor
sub TPlayer.production
  this.stone    += this.pStone
  this.crystal  += this.pCrystal
  this.creature += this.pCreature
end sub

type TEffect
  as TEffectType effectType    ' kind of effect
  as TEffectType effectTypeAlt ' kind of alternative effect
  as integer power, powerAlt   ' power of the effect
  as TCond relation, relatAlt  ' relation
  as TCond cond1, cond2        ' kind of condition
  as TComp comp                ' kind of comparsion
  as integer compValue         ' value to compare with
  as string desc, descAlt      ' optional description
  ' if (cond1 comp [cond2|compValue]) then effectType(power) else effectTypeAlt(powerAlt) 
end type

type TCard
  as string  cardname, desc
  as integer cost, costType, countEffects
  as TEffect effect(MAXEFFECTS-1)
  as integer special
  declare sub addEffect(e as TEffectType, p as integer, _
              cond1 as TCond = TCond.none, cond2 as TCond = TCond.none, comp as TComp = TComp.none, _
              v as integer = 0, relation as TCond = TCond.none)
end type
sub TCard.addEffect(e as TEffectType, p as integer, _
          cond1 as TCond = TCond.none, cond2 as TCond = TCond.none, comp as TComp = TComp.none, _
          v as integer = 0, relation as TCond = TCond.none)
  if this.countEffects < ubound(this.effect) then
    with this.effect(this.countEffects)
      .effectType = e
      .power      = p
      .cond1      = cond1
      .cond2      = cond2
      .comp       = comp
      .compValue  = v
      .relation   = relation
    end with
    this.countEffects += 1
  end if
end sub

type TDeck
  as TDeck ptr prev, nxt
  as integer card
  declare sub add(card as integer)
  declare sub del
end type
sub TDeck.add(card as integer)
  if this.nxt = 0 then
    this.prev = @this
    this.nxt  = @this
    this.card = card
  else
    dim as TDeck ptr newCard = new TDeck
    newCard->prev = @this
    newCard->nxt  = this.nxt
    newCard->card = card
    this.nxt->prev = newCard
    this.nxt = newCard
  end if
end sub
sub TDeck.del
  dim as TDeck ptr nxt = this.nxt
  if nxt = @this then
    this.prev = 0
    this.nxt  = 0
    this.card = 0
  else
    nxt->nxt->prev = @this
    this.nxt = nxt->nxt
    delete nxt
  end if
end sub

type TScore
  as integer move, score
end type

dim shared as TCard card()

declare sub calcEffect(player() as TPlayer, number as integer, effect as TEffect)
declare function calcRelation(player() as TPlayer, number as integer, value as integer, rel as TCond) as integer
declare function cardImg(c as TCard, player() as TPlayer, number as integer) as FB.Image ptr
declare sub center overload(x as integer, byref y as integer, s as string, c as uinteger = &hffffff, _
                   max as integer = 45, spacing as integer = 10)
declare sub center(img as FB.Image ptr, x as integer, byref y as integer, s as string, col as uinteger = &hffffff, _
                   max as integer = 45, spacing as integer = 10)
declare function colorOfEffect(eff as TEffectType) as uinteger
declare function getConditionValue(cond as string) as TCond
declare sub readCards(filename as string, language as string)
declare function readEffect(byref c as integer, xml as TypXML) as TEffectType
declare function removeNewline(text as string) as string
declare function testCondition(player() as TPlayer, number as integer, e as TEffect) as integer

sub calcEffect(player() as TPlayer, number as integer, effect as TEffect)
  dim p as integer, r as TCond, e as TEffectType
  with effect
    if testCondition(player(), number, effect) then
      e = .effectType
      p =  calcRelation(player(), number, .power, .relation)
    else
      e = .effectTypeAlt
      p =  calcRelation(player(), number, .powerAlt, .relatAlt)
    end if
    if r <> TCond.none then p = calcRelation(player(), number, p, r)
    select case e
      case TEffectType.stealStone
        if p > 0 then
          if player(3-number).stone < p then p = player(3-number).stone
        else
          if player(number).stone < -p then p = -player(number).stone
        end if
      case TEffectType.stealCrystal
        if p > 0 then
          if player(3-number).crystal < p then p = player(3-number).crystal
        else
          if player(number).crystal < -p then p = -player(number).crystal
        end if
      case TEffectType.stealCreature
        if p > 0 then
          if player(3-number).creature < p then p = player(3-number).creature
        else
          if player(number).creature < -p then p = -player(number).creature
        end if
      case TEffectType.stealStoneProd
        if p > 0 then
          if player(3-number).pStone < p+1 then p = player(3-number).pStone-1
        else
          if player(number).pStone < -p+1 then p = -player(number).pStone-1
        end if
      case TEffectType.stealCrystalProd
        if p > 0 then
          if player(3-number).pCrystal < p+1 then p = player(3-number).pCrystal-1
        else
          if player(number).pCrystal < -p+1 then p = -player(number).pCrystal-1
        end if
      case TEffectType.stealCreatureProd
        if p > 0 then
          if player(3-number).pCreature < p+1 then p = player(3-number).pCreature-1
        else
          if player(number).pCreature < -p+1 then p = -player(number).pCreature-1
        end if
    end select
    select case e
      case TEffectType.changeOwnStone         : player(number).stone       += p
      case TEffectType.changeOwnCrystal       : player(number).crystal     += p
      case TEffectType.changeOwnCreature      : player(number).creature    += p
      case TEffectType.changeOwnStoneProd     : player(number).pStone      += p
      case TEffectType.changeOwnCrystalProd   : player(number).pCrystal    += p
      case TEffectType.changeOwnCreatureProd  : player(number).pCreature   += p
      case TEffectType.changeOppStone         : player(3-number).stone     += p
      case TEffectType.changeOppCrystal       : player(3-number).crystal   += p
      case TEffectType.changeOppCreature      : player(3-number).creature  += p
      case TEffectType.changeOppStoneProd     : player(3-number).pStone    += p
      case TEffectType.changeOppCrystalProd   : player(3-number).pCrystal  += p
      case TEffectType.changeOppCreatureProd  : player(3-number).pCreature += p
      case TEffectType.changeBothStone        : player(number).stone       += p : player(3-number).stone     += p
      case TEffectType.changeBothCrystal      : player(number).crystal     += p : player(3-number).crystal   += p
      case TEffectType.changeBothCreature     : player(number).creature    += p : player(3-number).creature  += p
      case TEffectType.changeBothStoneProd    : player(number).pStone      += p : player(3-number).pStone    += p
      case TEffectType.changeBothCrystalProd  : player(number).pCrystal    += p : player(3-number).pCrystal  += p
      case TEffectType.changeBothCreatureProd : player(number).pCreature   += p : player(3-number).pCreature += p
      case TEffectType.stealStone             : player(number).stone       += p : player(3-number).stone     -= p
      case TEffectType.stealCrystal           : player(number).crystal     += p : player(3-number).crystal   -= p
      case TEffectType.stealCreature          : player(number).creature    += p : player(3-number).creature  -= p
      case TEffectType.stealStoneProd         : player(number).pStone      += p : player(3-number).pStone    -= p
      case TEffectType.stealCrystalProd       : player(number).pCrystal    += p : player(3-number).pCrystal  -= p
      case TEffectType.stealCreatureProd      : player(number).pCreature   += p : player(3-number).pCreature -= p
      case TEffectType.changeOwnWall          : player(number).wall        += p
      case TEffectType.changeOppWall          : player(3-number).wall      += p
      case TEffectType.changeBothWall         : player(number).wall        += p : player(3-number).wall   += p
      case TEffectType.changeOwnTower         : player(number).tower       += p
      case TEffectType.changeOppTower         : player(3-number).tower     += p
      case TEffectType.changeBothTower        : player(number).tower       += p : player(3-number).tower  += p
      case TEffectType.ownDamage
        if player(number).wall >= p then
          player(number).wall -= p
        else
          player(number).tower += player(number).wall - p
          player(number).wall = 0
        end if
      case TEffectType.oppDamage
        if player(3-number).wall >= p then
          player(3-number).wall -= p
        else
          player(3-number).tower += player(3-number).wall - p
          player(3-number).wall = 0
        end if
      case TEffectType.bothDamage
        if player(number).wall >= p then
          player(number).wall -= p
        else
          player(number).tower += player(number).wall - p
          player(number).wall = 0
        end if
        if player(3-number).wall >= p then
          player(3-number).wall -= p
        else
          player(3-number).tower += player(3-number).wall - p
          player(3-number).wall = 0
        end if
    end select
  end with
end sub

function calcRelation(player() as TPlayer, number as integer, value as integer, rel as TCond) as integer
  select case rel
    case ownStone         : return (value * player(number).stone)       \ 100
    case ownCrystal       : return (value * player(number).crystal)     \ 100
    case ownCreature      : return (value * player(number).creature)    \ 100
    case ownStoneProd     : return (value * player(number).pStone)      \ 100
    case ownCrystalProd   : return (value * player(number).pCrystal)    \ 100
    case ownCreatureProd  : return (value * player(number).pCreature)   \ 100
    case ownWall          : return (value * player(number).wall)        \ 100
    case ownTower         : return (value * player(number).tower)       \ 100
    case oppStone         : return (value * player(3-number).stone)     \ 100
    case oppCrystal       : return (value * player(3-number).crystal)   \ 100
    case oppCreature      : return (value * player(3-number).creature)  \ 100
    case oppStoneProd     : return (value * player(3-number).pStone)    \ 100
    case oppCrystalProd   : return (value * player(3-number).pCrystal)  \ 100
    case oppCreatureProd  : return (value * player(3-number).pCreature) \ 100
    case oppWall          : return (value * player(3-number).wall)      \ 100
    case oppTower         : return (value * player(3-number).tower)     \ 100
    case diffStone        : return (value * (player(number).stone     - player(3-number).stone))     \ 100
    case diffCrystal      : return (value * (player(number).crystal   - player(3-number).crystal))   \ 100
    case diffCreature     : return (value * (player(number).creature  - player(3-number).creature))  \ 100
    case diffStoneProd    : return (value * (player(number).pStone    - player(3-number).pStone))    \ 100
    case diffCrystalProd  : return (value * (player(number).pCrystal  - player(3-number).pCrystal))  \ 100
    case diffCreatureProd : return (value * (player(number).pCreature - player(3-number).pCreature)) \ 100
    case diffWall         : return (value * (player(number).wall      - player(3-number).wall))      \ 100
    case diffTower        : return (value * (player(number).tower     - player(3-number).tower))     \ 100
    case diffOwnBuilding  : return (value * (player(number).tower     - player(number).wall))        \ 100
    case diffOppBuilding  : return (value * (player(3-number).tower   - player(3-number).wall))      \ 100
  end select
  return value
end function

function cardImg(c as TCard, player() as TPlayer, number as integer) as FB.Image ptr
  dim as FB.Image ptr img = imagecreate(100, 140, 0)
  dim as uinteger col
  dim as integer posX = 51, posY = 65, spacing = 20, spec = 0
  dim as string tempString
  dim as TPlayer playerNew(1 to 2) = {player(1), player(2)}
  with c
    select case .costType
      case TCond.ownStone
        col = ColorStone
        playerNew(number).stone    -= .cost
      case TCond.ownCrystal
        col = ColorCrystal
        playerNew(number).crystal  -= .cost
      case TCond.ownCreature
        col = ColorCreature
        playerNew(number).creature -= .cost
      case else
        col = &hffffff
    end select
    line img, (2, 0)-step(96, 139), col, b
    line img, (3, 1)-step(92, 137), 0, bf
    center img, posX, 5, .cardname
    center img, posX, 25, getTranslation(l, "cost") & " " & .cost, col
    for i as integer = 0 to MAXEFFECTS - 1
      if .effect(i).effectType = TEffectType.none then continue for
      dim as string arr1 = "", arr2 = "", sign = "", text = ""
      dim as integer value = .effect(i).power
      dim as TEffectType effectType
      if .effect(i).cond1 <> TCond.none then spec = -1
      if testCondition(player(), number, .effect(i)) then
        effectType = .effect(i).effectType
        value = calcRelation(playerNew(), number, .effect(i).power, .effect(i).relation)
      else
        effectType = .effect(i).effectTypeAlt
        value = calcRelation(playerNew(), number, .effect(i).powerAlt, .effect(i).relatAlt)
      end if
      if value = 0 then continue for
      col = colorOfEffect(effectType)
      ' arrows
      select case effectType
        case changeOwnStone, changeOwnCrystal, changeOwnCreature, _
             changeOwnStoneProd, changeOwnCrystalProd, changeOwnCreatureProd
          if number = 1 then arr1 = "< " else arr2 = " >"
          if value > 0 then sign = "+"
        case changeOppStone, changeOppCrystal, changeOppCreature, _
             changeOppStoneProd, changeOppCrystalProd, changeOppCreatureProd
          if number = 1 then arr2 = " >" else arr1 = "< "
          if value > 0 then sign = "+"
        case changeBothStone, changeBothCrystal, changeBothCreature
          arr1 = "< " : arr2 = " >"
          if value > 0 then sign = "+"
        case changeBothStoneProd, changeBothCrystalProd, changeBothCreatureProd
          arr1 = "<" : arr2 = " >"
          if value > 0 then sign = "+"
        case changeOwnWall, changeOwnTower, ownDamage
          if number = 1 then arr1 = "< " else arr2 = " >"
        case changeOppWall, changeOppTower, oppDamage
          if number = 1 then arr2 = " >" else arr1 = "< "
        case changeBothWall, changeBothTower, bothDamage
          arr1 = "<" : arr2 = " >"
        case stealStone, stealCrystal, stealCreature, stealStoneProd, stealCrystalProd, stealCreatureProd
          if value > 0 xor number = 2 then
            arr1 = "< " : arr2 = " <"
          else
            arr1 = "> " : arr2 = " >"
          end if
          value = abs(value)
      end select
      ' additional text
      select case effectType
        case changeOwnStoneProd,  changeOwnCrystalProd,  changeOwnCreatureProd, _
             changeOppStoneProd,  changeOppCrystalProd,  changeOppCreatureProd, _
             changeBothStoneProd, changeBothCrystalProd, changeBothCreatureProd, _
             stealStoneProd,      stealCrystalProd,      stealCreatureProd
          text = " " & getTranslation(l, "prod")
        case changeOwnWall, changeOppWall, changeBothWall
          text = " " & getTranslation(l, "wall")
        case changeOwnTower, changeOppTower, changeBothTower
          text = " " & getTranslation(l, "tower")
        case ownDamage, oppDamage, bothDamage
          text = " " & getTranslation(l, "damage")
      end select
      center img, posX, posY, arr1 & sign & value & text & arr2, col
      posY += spacing
      calcEffect(playerNew(), number, .effect(i))
    next
    if spec then
      center img, posX, posY, getTranslation(l, "special")
      posY += spacing
    end if
    if .special and Special.again then
      select case .special and (Special.stone or Special.crystal or Special.creature)
        case Special.stone    : col = ColorStone
        case Special.crystal  : col = ColorCrystal
        case Special.creature : col = ColorCreature
        case else             : col = &hffffff
      end select
      center img, posX, posY, getTranslation(l, "againShort"), col
      posY += spacing
    end if
  end with
  return img
end function

sub center(x as integer, byref y as integer, s as string, col as uinteger = &hffffff, _
           max as integer = 45, spacing as integer = 10)
  center(0, x, y, s, col, max, spacing)
end sub

sub center(img as FB.Image ptr, x as integer, byref y as integer, s as string, col as uinteger = &hffffff, _
           max as integer = 45, spacing as integer = 10)
  dim as integer search = -1, plus
  dim as string tempString = s
  do while len(tempstring)
    plus = 2
    search = instr(tempString, "\n")
    if search = 0 then search = len(tempstring)+1
    if search > max + 1 then
      search = instrrev(tempstring, " ", max)
      if search = 0 then tempstring = left(tempstring, max) : search = max + 1
      plus = 1
    end if
    draw string img, (x - 4*(search-1), y), left(tempString, search-1), col
    tempString = mid(tempString, search + plus)
    if len(tempstring) then y += spacing
  loop
end sub

function colorOfEffect(eff as TEffectType) as uinteger
  select case eff
    case changeOwnStone,     changeOwnStoneProd,     changeOppStone,    changeOppStoneProd, _
        changeBothStone,    changeBothStoneProd,    stealStoneProd,    stealStone
      return ColorStone
    case changeOwnCrystal,   changeOwnCrystalProd,   changeOppCrystal,  changeOppCrystalProd, _
        changeBothCrystal,  changeBothCrystalProd,  stealCrystalProd,  stealCrystal
      return ColorCrystal
    case changeOwnCreature,  changeOwnCreatureProd,  changeOppCreature, changeOppCreatureProd, _
        changeBothCreature, changeBothCreatureProd, stealCreatureProd, stealCreature
      return ColorCreature
    case else
      return &hffffff
  end select
end function

function getConditionValue(cond as string) as TCond
  select case cond
    case "ownStone"                   : return TCond.ownStone
    case "ownCrystal"                 : return TCond.ownCrystal
    case "ownCreature"                : return TCond.ownCreature
    case "ownStoneProd"               : return TCond.ownStoneProd
    case "ownCrystalProd"             : return TCond.ownCrystalProd
    case "ownCreatureProd"            : return TCond.ownCreatureProd
    case "ownWall"                    : return TCond.ownWall
    case "ownTower"                   : return TCond.ownTower
    case "opponentStone"              : return TCond.oppStone
    case "opponentCrystal"            : return TCond.oppCrystal
    case "opponentCreature"           : return TCond.oppCreature
    case "opponentStoneProd"          : return TCond.oppStoneProd
    case "opponentCrystalProd"        : return TCond.oppCrystalProd
    case "opponentCreatureProd"       : return TCond.oppCreatureProd
    case "opponentWall"               : return TCond.oppWall
    case "opponentTower"              : return TCond.oppTower
    case "differenceStone"            : return TCond.diffStone
    case "differenceCrystal"          : return TCond.diffCrystal
    case "differenceCreature"         : return TCond.diffCreature
    case "differenceStoneProd"        : return TCond.diffStoneProd
    case "differenceCrystalProd"      : return TCond.diffCrystalProd
    case "differenceCreatureProd"     : return TCond.diffCreatureProd
    case "differenceWall"             : return TCond.diffWall
    case "differenceTower"            : return TCond.diffTower
    case "differenceOwnBuilding"      : return TCond.diffOwnBuilding
    case "differenceOpponentBuilding" : return TCond.diffOppBuilding
  end select
  return TCond.none
end function

sub readCards(filename as string, language as string)
  ' read cards
  dim as integer file = freefile
  open filename for input as #file
  dim as TypXml xml = xmlParse(xmlParse(, file).inhalt), xmltemp, xmlIf
  dim as string target
  dim as TCond cond1, cond2, relation
  dim comp as TComp, compValue as integer
  do while xml.inhalt <> ""
    if xml.tag <> "card" then continue do
    redim preserve card(ubound(card)+1)
    xmltemp = xmlParse(xml.inhalt)
    with card(ubound(card))
      do while xmltemp.tag <> ""
        compValue = 0
        select case xmltemp.tag
          case "name"
            if .cardname = "" or xmltemp.GetAttr("lang") = language then .cardname = xmltemp.inhalt
          case "costStone"
            .cost = valint(xmltemp.inhalt) : .costType = TCond.ownStone
          case "costCrystal"
            .cost = valint(xmltemp.inhalt) : .costType = TCond.ownCrystal
          case "costCreature"
            .cost = valint(xmltemp.inhalt) : .costType = TCond.ownCreature
          case "if"
            cond1 = getConditionValue(xmltemp.GetAttr("condition"))
            cond2 = getConditionValue(xmltemp.GetAttr("compareWith"))
            compValue = valint(xmltemp.GetAttr("compareWith"))
            select case xmltemp.GetAttr("relation")
              case "equal"   : comp = TComp.equal
              case "greater" : comp = TComp.greater
              case "lower"   : comp = TComp.lower
            end select
            xmlIf = xmlParse(xmltemp.inhalt)
            relation = getConditionValue(xmlIf.GetAttr("relation"))
            .addEffect readEffect(ubound(card), xmlIf), valint(xmlIf.inhalt), cond1, cond2, comp, compValue, relation
            xmlIf = xmlParse(xmlIf.rest)
            do while xmlIf.tag = "description"
              if .effect(.countEffects - 1).desc = "" orelse xmlIf.GetAttr("lang") = language then
                .effect(.countEffects - 1).desc = xmlIf.inhalt
              end if
              xmlIf = xmlParse(xmlIf.rest)
            loop
            if xmlIf.tag = "else" then
              xmlIf = xmlParse(xmlIf.rest)
              relation = getConditionValue(xmlIf.GetAttr("relation"))
              .effect(.countEffects - 1).effectTypeAlt = readEffect(ubound(card), xmlIf)
              .effect(.countEffects - 1).powerAlt      = valint(xmlIf.inhalt)
              .effect(.countEffects - 1).relatAlt      = relation
              xmlIf = xmlParse(xmlIf.rest)
              do while xmlIf.tag = "description"
                if .effect(.countEffects - 1).descAlt = "" orelse xmlIf.GetAttr("lang") = language then
                  .effect(.countEffects - 1).descAlt = xmlIf.inhalt
                end if
                xmlIf = xmlParse(xmlIf.rest)
              loop
            end if
        case "again" : .special or= Special.again
        case "againStone"    : .special or= Special.again or Special.stone
        case "againCrystal"  : .special or= Special.again or Special.crystal
        case "againCreature" : .special or= Special.again or Special.creature
        case "description"
          if .countEffects = 0 then
            .desc = xmltemp.inhalt
          elseif .effect(.countEffects - 1).desc = "" orelse xmltemp.GetAttr("lang") = language then
            .effect(.countEffects - 1).desc = xmltemp.inhalt
          end if
        case else
          relation = getConditionValue(xmltemp.GetAttr("relation"))
          .addEffect readEffect(ubound(card), xmltemp), valint(xmltemp.inhalt),,,,, relation
        end select
        xmltemp = xmlParse(xmltemp.rest)
      loop
      ' change 'damageXY' to 'changeXY'
      for i as integer = 0 to .countEffects - 1
        with .effect(i)
          if .effectType >= TEffectType.damageOwnWall and .effectType <= TEffectType.damageBothTower then
            .effectType    -= TEffectType.damageOwnWall - TEffectType.changeOwnWall
            .power         *= -1
          end if
          if .effectTypeAlt >= TEffectType.damageOwnWall and .effectTypeAlt <= TEffectType.damageBothTower then
            .effectTypeAlt -= TEffectType.damageOwnWall - TEffectType.changeOwnWall
            .powerAlt      *= -1
          end if
        end with
      next
    end with
    xml = xmlParse(xml.rest)
  loop
  close file
end sub

function readEffect(byref c as integer, xml as TypXML) as TEffectType
  with card(c)
    select case xml.tag
      case "addProductionStone"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnStoneProd
          case "enemy" : return TEffectType.changeOppStoneProd
          case "both"  : return TEffectType.changeBothStoneProd
        end select
      case "addProductionCrystal"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnCrystalProd
          case "enemy" : return TEffectType.changeOppCrystalProd
          case "both"  : return TEffectType.changeBothCrystalProd
        end select
      case "addProductionCreature"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnCreatureProd
          case "enemy" : return TEffectType.changeOppCreatureProd
          case "both"  : return TEffectType.changeBothCreatureProd
        end select
      case "addStone"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnStone
          case "enemy" : return TEffectType.changeOppStone
          case "both"  : return TEffectType.changeBothStone
        end select
      case "addCrystal"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnCrystal
          case "enemy" : return TEffectType.changeOppCrystal
          case "both"  : return TEffectType.changeBothCrystal
        end select
      case "addCreature"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnCreature
          case "enemy" : return TEffectType.changeOppCreature
          case "both"  : return TEffectType.changeBothCreature
        end select
      case "stealProductionStone"    : return TEffectType.stealStoneProd
      case "stealProductionCrystal"  : return TEffectType.stealCrystalProd
      case "stealProductionCreature" : return TEffectType.stealCreatureProd
      case "stealStone"              : return TEffectType.stealStone
      case "stealCrystal"            : return TEffectType.stealCrystal
      case "stealCreature"           : return TEffectType.stealCreature
      case "buildWall"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnWall
          case "enemy" : return TEffectType.changeOppWall
          case "both"  : return TEffectType.changeBothWall
        end select
      case "buildTower"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.changeOwnTower
          case "enemy" : return TEffectType.changeOppTower
          case "both"  : return TEffectType.changeBothTower
        end select
      case "damageWall"
        select case xml.GetAttr("target")
          case "self"  : return TEffectType.damageOwnWall
          case "enemy" : return TEffectType.damageOppWall
          case "both"  : return TEffectType.damageBothWall
        end select
      case "damageTower"
       select case xml.GetAttr("target")
          case "self"  : return TEffectType.damageOwnTower
          case "enemy" : return TEffectType.damageOppTower
          case "both"  : return TEffectType.damageBothTower
        end select
      case "damage"
       select case xml.GetAttr("target")
          case "self"  : return TEffectType.ownDamage
          case "enemy" : return TEffectType.oppDamage
          case "both"  : return TEffectType.bothDamage
        end select
      case else
        print "unknown tag <" & xml.tag & ">"
    end select
  end with
end function

function removeNewline(text as string) as string
  dim ret as string = text, search as integer = instr(ret, "\n")
  do while search
    if mid(ret, search - 1, 1) = "-" then
      ret = left(ret, search - 2) & mid(ret, search + 2)
    else
      ret = left(ret, search - 1) & " " & mid(ret, search + 2)
    end if
    search = instr(search - 1, ret, "\n")
  loop
  return ret
end function

function testCondition(player() as TPlayer, number as integer, e as TEffect) as integer
  if e.comp = TComp.none then return -1
  dim as TComp comp = e.comp
  dim as integer cond1, cond2
  cond1 = calcRelation(player(), number, 100, e.cond1)
  cond2 = iif(e.cond2 = TCond.none, e.compValue, calcRelation(player(), number, 100, e.cond2))
  select case comp
    case TComp.equal   : if cond1 <> cond2 then return 0
    case TComp.greater : if cond1 <= cond2 then return 0
    case TComp.lower   : if cond1 >= cond2 then return 0
  end select
  return -1
end function
